@@ -1,7 +1,7 @@
-{-# LANGUAGE DeriveGeneric, OverloadedStrings #-}
+{-# LANGUAGE DeriveGeneric #-}
 -- | Simple English clause creation parameterized by individual words.
 module NLP.Miniutter.English
-  ( Part(..), Person(..), Polarity(..), Irregular
+  ( Part(..), Person(..), Polarity(..), Irregular(..)
   , makeSentence, makePhrase, defIrregular, (<+>)
   ) where
 
@@ -20,12 +20,21 @@ import           NLP.Minimorph.Util hiding ((<>))
 -- | Various basic and compound parts of English simple present tense clauses.
 -- Many of the possible nestings do not make sense. We don't care.
 data Part =
-    String !String      -- ^ handle for a String parameter
-  | Text !Text          -- ^ handle for a Text parameter
+    String !String      -- ^ handle for a @String@ parameter
+  | Text !Text          -- ^ handle for a @Text@ parameter
   | Cardinal !Int       -- ^ cardinal number, spelled in full up to 10
+  | Car !Int            -- ^ cardinal number, not spelled
   | Ws !Part            -- ^ plural form of a phrase
-  | CarWs !Int !Part       -- ^ plural prefixed with a cardinal, not spelled
-  | CardinalWs !Int !Part  -- ^ plural prefixed with a cardinal, spelled
+  | CardinalAWs !Int !Part
+                        -- ^ plural prefixed with a cardinal, spelled,
+                        --   with \"a\" for 1 and \"no\" for 0
+  | CardinalWs !Int !Part
+                        -- ^ plural prefixed with a cardinal, spelled
+  | CarAWs !Int !Part   -- ^ plural prefixed with a cardinal, not spelled,
+                        --   with \"a\" for 1 and \"no\" for 0
+  | CarWs !Int !Part    -- ^ plural prefixed with a cardinal, not spelled;
+  | Car1Ws !Int !Part   -- ^ plural prefixed with a cardinal, not spelled;
+                        --   no prefix at all for 1
   | Ordinal !Int        -- ^ ordinal number, spelled in full up to 10
   | Ord !Int            -- ^ ordinal number, not spelled
   | AW !Part            -- ^ phrase with indefinite article
@@ -33,20 +42,19 @@ data Part =
   | WWxW !Part ![Part]  -- ^ collection
   | Wown !Part          -- ^ non-premodifying possesive
   | WownW !Part !Part   -- ^ attributive possesive
-  | Append !Part !Part  -- ^ no space in between
-  | !Part :> !Part      -- ^ no space in between  -- deprecated, use <>
+  | Append !Part !Part  -- ^ no space in between; one can also just use @<>@
   | Phrase ![Part]      -- ^ space-separated sequence
   | Capitalize !Part    -- ^ make the first letter into a capital letter
   | SubjectVerb !Person !Polarity !Part !Part
                         -- ^ conjugation according to polarity,
-                        -- with a default person (pronouns override it)
+                        --   with a default person (pronouns override it)
   | SubjectVerbSg !Part !Part
-                        -- ^ a shorthand for Sg3rd and Yes
+                        -- ^ a shorthand for @Sg3rd@ and @Yes@
   | SubjectVVxV !Part !Person !Polarity !Part ![Part]
                         -- ^ conjugation of all verbs according to polarity,
-                        -- with a default person (pronouns override it)
+                        --   with a default person (pronouns override it)
   | SubjectVVandVSg !Part ![Part]
-                        -- ^ a shorthand for "and", Sg3rd and Yes
+                        -- ^ a shorthand for \"and\", @Sg3rd@ and @Yes@
   deriving (Show, Eq, Ord, Generic)
 
 instance Binary Part
@@ -82,11 +90,15 @@ instance Binary Polarity
 
 -- | Nouns with irregular plural form and nouns with irregular indefinite
 -- article.
-type Irregular = (Map Text Text, Map Text Text)
+data Irregular = Irregular
+  { irrPlural     :: Map Text Text
+  , irrIndefinite :: Map Text Text
+  }
 
 -- | Default set of words with irregular forms.
 defIrregular :: Irregular
-defIrregular = (defIrrPlural, defIrrIndefinite)
+defIrregular =
+  Irregular {irrPlural = defIrrPlural, irrIndefinite = defIrrIndefinite}
 
 -- | Realise a complete sentence, capitalized, ending with a dot.
 makeSentence :: Irregular -> [Part] -> Text
@@ -106,11 +118,20 @@ makePart irr part = case part of
   String t -> T.pack t
   Text t -> t
   Cardinal n -> cardinal n
+  Car n -> tshow n
   Ws p -> onLastWord (makePlural irr) (mkPart p)
-  CarWs 1 p -> mkPart (AW p)  -- TOOD: a variant without 'a'
-  CarWs n p -> T.pack (show n) <+> onLastWord (makePlural irr) (mkPart p)
-  CardinalWs 1 p -> mkPart (AW p)
+  CardinalAWs 0 p -> "no" <+> onLastWord (makePlural irr) (mkPart p)
+  CardinalAWs 1 p -> mkPart (AW p)
+  CardinalAWs n p -> cardinal n <+> onLastWord (makePlural irr) (mkPart p)
+  CardinalWs 1 p -> cardinal 1 <+> mkPart p  -- spelled number
   CardinalWs n p -> cardinal n <+> onLastWord (makePlural irr) (mkPart p)
+  CarAWs 0 p -> "no" <+> onLastWord (makePlural irr) (mkPart p)
+  CarAWs 1 p -> mkPart (AW p)
+  CarAWs n p -> tshow n <+> onLastWord (makePlural irr) (mkPart p)
+  CarWs 1 p -> "1" <+> mkPart p
+  CarWs n p -> tshow n <+> onLastWord (makePlural irr) (mkPart p)
+  Car1Ws 1 p -> mkPart p  -- no number, article, anything; useful
+  Car1Ws n p -> tshow n <+> onLastWord (makePlural irr) (mkPart p)
   Ordinal n -> ordinal n
   Ord n -> ordinalNotSpelled n
   AW p -> onFirstWord (addIndefinite irr) (mkPart p)
@@ -124,7 +145,6 @@ makePart irr part = case part of
   WownW p1 p2 -> onLastWord attributive (mkPart p1) <+> mkPart p2
   Phrase lp -> makePhrase irr lp
   Append p1 p2 -> mkPart p1 <> mkPart p2
-  p1 :> p2 -> mkPart p1 <> mkPart p2
   Capitalize p -> capitalize $ mkPart p
   SubjectVerb defaultPerson Yes s v ->
     subjectVerb defaultPerson (mkPart s) (mkPart v)
@@ -188,13 +208,13 @@ capitalize t = case T.uncons t of
   Just (c, rest) -> T.cons (toUpper c) rest
 
 makePlural :: Irregular -> Text -> Text
-makePlural (irrPlural, _) t =
+makePlural Irregular{irrPlural} t =
   case Map.lookup t irrPlural of
     Just u  -> u
     Nothing -> defaultNounPlural t
 
 addIndefinite :: Irregular -> Text -> Text
-addIndefinite (_, irrIndefinite) t =
+addIndefinite Irregular{irrIndefinite} t =
   case Map.lookup t irrIndefinite of
     Just u  -> u <+> t
     Nothing -> indefiniteDet t <+> t
@@ -388,6 +408,8 @@ defIrrPlural = Map.fromList
   , ("hovercraft",  "hovercraft")
   , ("information", "information")
   , ("Information", "Information")
+  , ("whiff",       "whiffs")
+  , ("graffiti",    "graffiti")
   ]
 
 -- | Default set of nouns with irregular indefinite article.
